#//-----------------------------------------------
#//TUSB-0224ADM用Pythonサンプルプログラム
#//
#//2024_04_08　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import TUSB0224ADMMod       #TUSB-0224ADM ドライバ呼び出しモジュール
import tkinter as tk
from functools import partial
import threading
import time
import datetime
from SpFuncs import *

# color define 入力値表示用
COL_RED = "#FFF000000"      #赤
COL_DRED = "#800000000"     #暗い赤

# Sub Functions /////////////////////
# Widgetの表示を設定する
# stat -> 'Close' , 'OpenStop' , 'OpenRun'
def WidgetState(stat):
    """フォームの表示を切り替える処理
    
    Arguments
    stat            ----    設定状態を表す文字列

    'Close' - デバイス非接続時
    'OpenStop' - デバイスが接続されていて連続取り込み実行していない時
    'OpenRun' - デバイスが接続されていて連続取り込みを実行している時
    """
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOpenStop = tk.NORMAL if( stat == 'OpenStop' ) else tk.DISABLED
    EnOpenRun = tk.NORMAL if( stat == 'OpenRun' ) else tk.DISABLED
    Enrty_SN['state'] = EnClose
    Btn_Connect['state'] = EnClose
    Btn_DisConnect['state'] = EnOpenStop
    Btn_SyncModeSet['state'] = EnOpenStop
    Btn_Sync['state'] = EnOpenStop
    Btn_IntClkSet['state']=EnOpenStop
    for i in range(2):
        RBtn_Sync[i]['state'] = EnOpenStop
    for ch in range(2):
        for i in range(2):
            RBtn_Attn[ch][i]['state'] = EnOpenStop
            RBtn_Couple[ch][i]['state'] = EnOpenStop
        for i in range(3):
            RBtn_Ampl[ch][i]['state'] = EnOpenStop
    Btn_TrgSet['state'] = EnOpenStop
    Btn_Start['state'] = EnOpenStop
    Btn_Stop['state'] = EnOpenRun
    Btn_Trg['state'] = EnOpenRun if TrgTypeDict["ソフトウェア"] == ComBox_TrgType.current() else tk.DISABLED
    Btn_DoutSet['state']=EnOpenStop
    Btn_DinRead['state']=EnOpenStop
    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

# タイマースレッド管理クラス ////////////////////////////////////
#厳密なインターバルではない。関数処理が終わったら指定時間待つだけ。
#from builtins import function
class intTT():
    """タイマースレッド管理クラス
    厳密なインターバル処理ではない。
    関数処理が終わったら指定時間待って再度実行しつづるループ。
    """
    thread :threading.Thread = None # スレッドクラスオブジェクト
    thread_en : bool = None         # スレッド実行イネーブルフラグ
    call_func = None                # ループで実行する関数
    interval = None                 # ループの待ち時間

    def __init__(self,inttime,callf):
        """初期化"""
        self.interval = inttime # インターバルタイム
        self.call_func = callf  # 処理関数登録
    
    def ThreadF(self):
        """スレッド関数(ループ)"""
        while self.thread_en:
            self.call_func()
            time.sleep(self.interval)

    def Start(self):
        """スレッド開始処理"""
        if self.thread == None:
            self.thread_en = True
            self.thread = threading.Thread(target=self.ThreadF)
            self.thread.start()
    
    def Stop(self):
        """スレッド停止処理"""
        if self.thread_en != None:
            self.thread_en = False
            self.thread.join(self.interval+0.1)
            self.thread = None 

#計測タイマー処理  ////////////////////////////////////

class _measdata():
    wfile = [None,None]     #IOオブジェクト[FileCh1,FileCh2]
    dcnt = [0,0]            #取り込み済みデータ数[Ch1,Ch2]
meas_d = _measdata()

statusstr = {
    0:"停止中",
    1:"プレトリガ長待機",
    2:"トリガ待ち",
    3:"サンプリング中",
    4:"データオーバフロー"
}

def Meas():
    """計測中定周期処理(タイマースレッドから呼び出される)"""
    for ch in range(2):
        try:
            (status,datalen) = AdMod.Smpl_Status(ch)
        except Exception as e:
            Disp_Err['text'] = str(e)
            return
        if datalen > 0:
            data = AdMod.Smpl_Data(ch,datalen)
            for sdat in data:
                meas_d.wfile[ch].write(str(sdat)+"\n")
            meas_d.dcnt[ch] += len(data)
            Disp_State[ch]['text'] = statusstr[status] + "Data長 = " + str(meas_d.dcnt[ch])

meas_t = intTT(0.1,Meas)

def CMeas_Start():
    """計測の開始"""
    ftimestr = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    meas_d.wfile[0] = open(ftimestr+"_Ch1.csv","w")
    meas_d.wfile[1] = open(ftimestr+"_Ch2.csv","w")
    meas_d.dcnt = [0,0]
    meas_t.Start()

def CMeas_Stop():
    """計測の終了"""
    meas_t.Stop()
    AdMod.Smpl_Stop()
    meas_d.wfile[0].close()
    meas_d.wfile[1].close()

# モニタータイマー処理 ////////////////////////////////////
def Monitor():
    ret = AdMod.Ad_Sample()
    Disp_VoltCh1["text"] = format(ret[0],".6f")
    Disp_VoltCh2["text"] = format(ret[1],".6f")

mon_t = intTT(0.5,Monitor)

# AD 入力状態確認
def AdInputCheck():
    for ch in range(2):
        ret = AdMod.Ad_InputChk(ch)
        RBtnV_Attn[ch].set(ret[0])
        RBtnV_Couple[ch].set(ret[1])
        RBtnV_Ampl[ch].set(ret[2])


#/// Button event handlers /////////////////////////

AdMod :TUSB0224ADMMod.TUSB0224ADM_Mod = None
def Connect_on():
    """デバイス接続"""
    global AdMod
    if AdMod != None:
        return
    try:
        AdMod = TUSB0224ADMMod.TUSB0224ADM_Mod(ForceToInt(Enrty_SN.get()))
        WidgetState('OpenStop')
        AdInputCheck()
        mon_t.Start()
    except Exception as e:
        ShowInfo(e)

def DisConnect_on():
    """デバイス接続解放"""
    global AdMod
    if AdMod != None:
        mon_t.Stop()
        WidgetState('Close')
        AdMod.DevClose()
        AdMod = None

def SyncModeSet_on():
    """同期モード設定"""
    try:
        AdMod.SyncMode(ComBox_SyncMode.current())
    except Exception as e:
        ShowInfo(e)

def SyncType_on():
    """同期状態設定"""
    try:
        AdMod.SyncAdj(RBtnV_SyncType.get())
    except Exception as e:
        ShowInfo(e)

def SyncAdj_on():
    """同期処理実行"""
    try:
        AdMod.SyncAdj(2)
    except Exception as e:
        ShowInfo(e)

def IntClkSet_on():
    """内部ADクロック設定"""
    ClkSrc = ComBox_BaseClk.current()
    ClkDiv = SpnBox_DivRate.get()
    Filter = RBtnV_Filter.get()
    try:
        AdMod.Ad_IntClock(ClkSrc,ClkDiv,Filter)
    except Exception as e:
        ShowInfo(e)

def IntClkDisp(event=None):
    """サンプリングレート更新事象"""
    ClkSrc = ComBox_BaseClk.current()
    ClkDiv = SpnBox_DivRate.get()
    try:
        Disp_SmplRate['text'] = GenSampleRateStr(ClkSrc,ClkDiv+1)
    except Exception as e:
        ShowInfo(e)

def InputChange_on(Ch):
    """AD入力設定"""
    attn = RBtnV_Attn[Ch].get()
    couple = RBtnV_Couple[Ch].get()
    ampl = RBtnV_Ampl[Ch].get()
    try:
        AdMod.Ad_Input(Ch,attn,couple,ampl)
    except Exception as e:
        ShowInfo(e)

def TrgSet_on():
    """トリガ設定"""
    trgtype = ComBox_TrgType.current()
    trgch = RBtnV_Trgch.get()
    if trgtype == (0 or 1): # "アナログ立上り" or "アナログ立下り"
        level1 = SpnBox_TrgLev.get()
        level2 = SpnBox_Hys.get()
    elif trgtype == (2 or 3): # "範囲内" or "範囲外"
        level1 = SpnBox_Upper.get()
        level2 = SpnBox_Lower.get()
    else:
        level1 = 0
        level2 = 0
    try:
        AdMod.Smpl_TrgLevel(trgtype,trgch,level1,level2)
    except Exception as e:
        ShowInfo(e)

def SmplStart_on():
    """サンプリング開始"""
    pretrglen = SpnBox_PreLen.get()
    AdMod.Smpl_Start(pretrglen,0)
    try:
        CMeas_Start()
    except OSError as e:
        ShowInfo(e)
    else:
        WidgetState('OpenRun')

def SmplStop_on():
    """サンプリング停止"""
    CMeas_Stop()
    try:
        AdMod.Smpl_Stop()
    except Exception as e:
        ShowInfo(e)
    WidgetState('OpenStop')

def SmplTrg_on():
    """ソフトウェアトリガ発生"""
    try:
        AdMod.Smpl_SoftTrigger()
    except Exception as e:
        ShowInfo(e)

def DoutSet_on():
    """ディジタル出力設定"""
    outdata = 0
    for bit in range(4):
        outdata += (1<<bit) if ChkBox_DOUT[bit].get() else 0
    try:
        AdMod.Dio_Out(outdata)
    except Exception as e:
        ShowInfo(e)

def DinRead_on():
    """ディジタル入力読み込み"""
    try:
        ret = AdMod.Dio_In()
    except Exception as e:
        ShowInfo(e)
    else:
        for bit in range(4):
            if bitChk(ret,bit):
                Led_DIN[bit]['bg'] = "#FFF000000" #Red
            else:
                Led_DIN[bit]['bg'] = "#800000000" #Dark Red

#/// form generator ///////////////////////////
#フォーム生成
form = tk.Tk()
form.geometry('710x665')
form.title('TUSB-0224ADM Sample Program')

#widget instance holders
LabelFrame = [None for _ in range(11)]

# シリアル番号　Connect DisConnect　ボタン
GetLabel(form,'Serial number',0.007,0.03)
Enrty_SN = GetEntry(form,"0",0.1,0.14,0.03)
Btn_Connect = GetButton(form,"Connect",Connect_on,0.1,0.35,0.03)
Btn_DisConnect = GetButton(form,"DisConnect",DisConnect_on,0.1,0.5,0.03)

#同期モードフレーム
SyncModeDict= {
    "単独動作":0,
    "同期動作(マスター)":1,
    "同期動作(スレーブ)":2,
}
LabelFrame[0] = GetLFrame(form,'同期モード',0.58,0.065,0.007,0.06)
ComBox_SyncMode = GetComboBox(LabelFrame[0],list(SyncModeDict.keys()),0.3,0.1,0.35)
Btn_SyncModeSet = GetButton(LabelFrame[0],"Set",SyncModeSet_on,0.2,0.85,0.35)

#ADクロック同期フレーム
SyncTypeDict= {
    "通常動作":0,
    "調整待機":1,
}
RBtnV_SyncType = tk.IntVar(value=0)
LabelFrame[1] = GetLFrame(form,'ADクロック同期',0.58,0.065,0.007,0.13)
RBtn_Sync :list = GetRadioButtons(LabelFrame[1],SyncTypeDict,0.05,0.4,0.25,0.0,RBtnV_SyncType,SyncType_on)
Btn_Sync = GetButton(LabelFrame[1],"調整実行(単独 or マスター)",SyncAdj_on,0.4,0.75,0.35)

#内部ADクロックフレーム
BaseClkDict= {
    "512kSPS":0,
    "256kSPS":1,
    "128kSPS":2,
    "64kSPS":3,
    "400kSPS":4,
    "200kSPS":5,
    "100kSPS":6,
    "50kSPS":7,
    "352.865kSPS":8,
    "176.432kSPS":9,
    "88.2162kSPS":10,
    "44.1081kSPS":11,
    "384kSPS":12,
    "192kSPS":13,
    "96kSPS":14,
    "48kSPS":15,
    "外部/32":16,
    "外部/64":17,
    "外部/128":18,
    "外部/256":19,
}
FilterDict= {
    "Filter 1":0,
    "Filter 2":1,
}
RBtnV_Filter = tk.IntVar(value=0)
SpnBox_DivRate = tk.IntVar(value=0)
LabelFrame[2] = GetLFrame(form,'内部ADクロック',0.58,0.174,0.007,0.202)
GetLabel(LabelFrame[2],"ベースレート",0.05,0.2)
GetLabel(LabelFrame[2],"分周比[n]",0.05,0.5)
GetLabel(LabelFrame[2],"サンプリングレート",0.05,0.8)
ComBox_BaseClk = GetComboBox(LabelFrame[2],list(BaseClkDict.keys()),0.3,0.3,0.2,IntClkDisp)
Spin_DivRate = GetSpinBoxN(LabelFrame[2],0,255,1,SpnBox_DivRate,0.3,0.3,0.5,IntClkDisp)
GetRadioButtons(LabelFrame[2],FilterDict,0.7,0.2,0.0,0.2,RBtnV_Filter)
Disp_SmplRate = GetLabelDisp(LabelFrame[2],"",0.35,0.3,0.8)
Btn_IntClkSet = GetButton(LabelFrame[2],"Set",IntClkSet_on,0.2,0.85,0.8)

#ADCh1/Ch2 入力設定フレーム
AttnDict={
    "*1":0,
    "*1/10":1,
}
CoupleDict={
    "DC":0,
    "AC":1,
}
AmpDict={
    "*1":0,
    "*3":1,
    "*10":2,
}
RBtn_Attn = list()
RBtn_Couple = list()
RBtn_Ampl = list()
RBtnV_Attn = [tk.IntVar(value=0) for _ in range(2)]
RBtnV_Couple = [tk.IntVar(value=0) for _ in range(2)]
RBtnV_Ampl = [tk.IntVar(value=0) for _ in range(2)]
LabelFrame[3] = GetLFrame(form,'ADCh1入力設定',0.286,0.164,0.007,0.383)
LabelFrame[4] = GetLFrame(form,'ADCh2入力設定',0.286,0.164,0.303,0.383)
for ch in range(2):
    fnum = 3 + ch
    GetLabel(LabelFrame[fnum],"減衰器",0.05,0.2)
    GetLabel(LabelFrame[fnum],"結合",0.05,0.45)
    GetLabel(LabelFrame[fnum],"増幅器",0.05,0.7)
    RBtn_Attn.append(GetRadioButtons(LabelFrame[fnum],AttnDict,0.3,0.2,0.2,0.0,RBtnV_Attn[ch],command = partial(InputChange_on,Ch=ch)))
    RBtn_Couple.append(GetRadioButtons(LabelFrame[fnum],CoupleDict,0.3,0.45,0.2,0.0,RBtnV_Couple[ch],command = partial(InputChange_on,Ch=ch)))
    RBtn_Ampl.append(GetRadioButtons(LabelFrame[fnum],AmpDict,0.3,0.7,0.2,0.0,RBtnV_Ampl[ch],command = partial(InputChange_on,Ch=ch)))

#トリガ設定フレーム
TrgTypeDict = {
    "アナログ立上り":0,
    "アナログ立下り":1,
    "範囲内":2,
    "範囲外":3,
    "デジタル立上り":4,
    "デジタル立下り":5,
    "ソフトウェア":6,
}
TrgChDict={
    "Ch1":0,
    "Ch2":1,
}
RBtnV_Trgch = tk.IntVar(value=0)
SpnBox_TrgLev = tk.IntVar(value=0)
SpnBox_Hys = tk.IntVar(value=10000)
SpnBox_Upper = tk.IntVar(value=0)
SpnBox_Lower = tk.IntVar(value=0)
LabelFrame[5] = GetLFrame(form,'トリガ設定',0.58,0.203,0.007,0.558)
GetLabel(LabelFrame[5],"トリガタイプ",0.05,0.15)
GetLabel(LabelFrame[5],"アナログトリガレベル",0.05,0.35)
GetLabel(LabelFrame[5],"アナログトリガ範囲上",0.05,0.55)
GetLabel(LabelFrame[5],"アナログチャンネル",0.5,0.15)
GetLabel(LabelFrame[5],"ヒステリシス",0.5,0.35)
GetLabel(LabelFrame[5],"アナログトリガ範囲下",0.5,0.55)
ComBox_TrgType = GetComboBox(LabelFrame[5],list(TrgTypeDict.keys()),0.22,0.25,0.15)
GetRadioButtons(LabelFrame[5],TrgChDict,0.75,0.15,0.12,0.0,RBtnV_Trgch)
GetSpinBoxN(LabelFrame[5],-8388608,8388607,1,SpnBox_TrgLev,0.17,0.33,0.35)
GetSpinBoxN(LabelFrame[5],-8388608,8388607,1,SpnBox_Hys,0.17,0.78,0.35)
GetSpinBoxN(LabelFrame[5],-8388608,8388607,1,SpnBox_Upper,0.17,0.33,0.55)
GetSpinBoxN(LabelFrame[5],-8388608,8388607,1,SpnBox_Lower,0.17,0.78,0.55)
Btn_TrgSet = GetButton(LabelFrame[5],"Set",TrgSet_on,0.2,0.85,0.8)

#サンプリングフレーム
SpnBox_PreLen = tk.IntVar(value=0)
Disp_State = list()
LabelFrame[6] = GetLFrame(form,'サンプリング',0.58,0.192,0.007,0.773)
GetLabel(LabelFrame[6],"プレトリガ",0.0,0.35)
GetLabel(LabelFrame[6],"状態(Ch1)",0.0,0.6)
GetLabel(LabelFrame[6],"状態(Ch2)",0.0,0.85)
Btn_Start = GetButton(LabelFrame[6],"開始",SmplStart_on,0.2,0.3,0.1)
Btn_Stop = GetButton(LabelFrame[6],"停止",SmplStop_on,0.2,0.5,0.1)
Btn_Trg = GetButton(LabelFrame[6],"トリガ",SmplTrg_on,0.2,0.85,0.1)
GetSpinBoxN(LabelFrame[6],0,1048576,1,SpnBox_PreLen,0.25,0.2,0.35)
Disp_Err = GetLabelDisp(LabelFrame[6],"",0.45,0.5,0.35)
Disp_State.append(GetLabelDisp(LabelFrame[6],"",0.75,0.2,0.6))
Disp_State.append(GetLabelDisp(LabelFrame[6],"",0.75,0.2,0.85))

#ディジタル出力フレーム
ChkBox_DOUT = [tk.BooleanVar(value=False) for _ in range(4)]
LabelFrame[7] = GetLFrame(form,'ディジタル出力',0.348,0.117,0.597,0.06)
for bit in range(4):
    GetChkBox(LabelFrame[7],str(bit),bit*0.2,0.2,ChkBox_DOUT[bit])
Btn_DoutSet = GetButton(LabelFrame[7],"Set",DoutSet_on,0.3,0.8,0.65)

#ディジタル入力フレーム
Led_DIN = list()
LabelFrame[8] = GetLFrame(form,'ディジタル入力',0.348,0.117,0.597,0.186)
for bit in range(4):
    Led_DIN.append(GetLedCanvas(LabelFrame[8],COL_DRED,0.05+bit*0.2,0.2))
    GetLabel(LabelFrame[8],str(bit),0.1+bit*0.2,0.2)
Btn_DinRead = GetButton(LabelFrame[8],"Read",DinRead_on,0.3,0.8,0.65)

#入力電圧値フレーム
LabelFrame[9] = GetLFrame(form,'入力電圧値',0.348,0.117,0.597,0.314)
GetLabel(LabelFrame[9],"Ch1",0.05,0.3)
GetLabel(LabelFrame[9],"Ch2",0.05,0.7)
GetLabel(LabelFrame[9],"[V]",0.6,0.3)
GetLabel(LabelFrame[9],"[V]",0.6,0.7)
Disp_VoltCh1 = GetLabelDisp(LabelFrame[9],"",0.4,0.2,0.3)
Disp_VoltCh2 = GetLabelDisp(LabelFrame[9],"",0.4,0.2,0.7)

# 校正数値フレーム(本プログラムでは実装しない)
# LabelFrame[10] = GetLFrame(form,'校正数値',0.348,0.517,0.597,0.445)
# GetLabel(LabelFrame[10],"Ch1",0.05,0.03)
# GetLabel(LabelFrame[10],"Ch2",0.05,0.53)
# MLDisp_CalibCh1 = GetLabelDispML(LabelFrame[10],"",0.7,0.45,0.25,0.01)
# MLDisp_CalibCh2 = GetLabelDispML(LabelFrame[10],"",0.7,0.45,0.25,0.51)

#//// Start

WidgetState('Close')
IntClkDisp()
form.mainloop()     # フォーム画面表示
